package extension

import (
	"encoding/json"
	"reflect"
	"testing"

	"git.perx.ru/perxis/perxis-go/pkg/collections"
	"git.perx.ru/perxis/perxis-go/pkg/schema"
	"git.perx.ru/perxis/perxis-go/pkg/schema/validate"
	"github.com/stretchr/testify/require"
)

func Test_getEnumOpt(t *testing.T) {
	tests := []struct {
		name string
		opts map[int32]string
		want []validate.EnumOpt
	}{
		{
			name: "#1",
			opts: map[int32]string{1: "a", 2: "b", 3: "c", 1000: "w"},
			want: []validate.EnumOpt{
				{Name: "a", Value: float64(1)},
				{Name: "b", Value: float64(2)},
				{Name: "c", Value: float64(3)},
				{Name: "w", Value: float64(1000)},
			},
		},
	}
	for _, tt := range tests {
		t.Run(tt.name, func(t *testing.T) {
			if got := getEnumOpt(tt.opts); !reflect.DeepEqual(got, tt.want) {
				t.Errorf("getEnumOpt() = %v, want %v", got, tt.want)
			}
		})
	}
}

func TestEqualSchema(t *testing.T) {
	for _, collection := range []*collections.Collection{
		NewActionsCollection("", ""),
		NewExtensionsCollection("", ""),
	} {
		s1 := collection.Schema
		b, err := s1.MarshalJSON()
		require.NoError(t, err)
		s2 := schema.New()
		err = json.Unmarshal(b, s2)
		s1.ClearState()
		s2.ClearState()
		require.NoError(t, err)
		require.Equal(t, s1.Field, s2.Field)
	}
}
